'use strict';

const {findVLNV} = require('duh-core');

const perBusInterface = (catalog, parent) => bi => {
  const path = parent + '.' + bi.name;
  console.log('; busInterface: ' + path);
};

const assertSchema = (path, schema) => {
  console.log('; schema of ' + path);

  Object.keys(schema).map(key => {

    const val = schema[key];
    const v = path + '.' + key;

    if (val.type === 'integer') {
      console.log('(declare-const ' + v +' Int)');
    } else if (val.type === 'number') {
      console.log('(declare-const ' + v +' Real)');
    } else {
      console.log('; ' + path + '.' + key);
    }

    if (val.minimum !== undefined) {
      console.log('(assert (>= ' + v + ' ' + val.minimum + '))');
    }
    if (val.maximum !== undefined) {
      console.log('(assert (<= ' + v + ' ' + val.maximum + '))');
    }

    if (val.enum !== undefined) {
      console.log(
        '(assert (or ' +
          val.enum.map(e => '(= ' + v + ' ' + e + ')').join(' ') +
        ')))'
      );
    }

    console.log('');
  });
};

const perInstance = (catalog, parent) => e => {
  const path = parent + '.' + e.name;
  const component = findVLNV(catalog.components, 'component', e.ref);
  console.log('; instance: ' + path + ' of component: ' + component.name);
  assertSchema(path, component.pSchema || {});
  (component.busInterfaces || []).map(perBusInterface(catalog, path));
};

const perConnection = (catalog, parent) => e => {
  const path = parent + '.' + e.name;
  console.log('; connection: ' + path);
};

const perDesign = catalog => e => {
  const design = e.design;
  const path = design.name;
  console.log('; design: ' + path);
  design.instances.map(perInstance(catalog, path));
  design.connections.map(perConnection(catalog, path));
  console.log('');
};

function checker (duh) {
  const catalog = duh.catalog;
  if (catalog !== undefined) {
    (catalog.designs || []).map(perDesign(catalog));
  }
}

module.exports = checker;
